;
;Tic-Tac-Toe using the IR Link for TI89.
;
;By Sami Khawam(sKhawam@bigfoot.com)
;
;
;
;This is a very simple 2 players Tic-Tac-Toe game that
;demonstrate the use of the IR Link. However, the game
;can be played with the cable and without the IR Link.
;
;The "Linking without Ack" mode should be used when
;running this program. For more info on how
;to build and use the IR Link see my homepage:
;
;  http://sami.ticalc.org
;
;
;The program is very simple to use. The keys used :
;
;Keys 9...1      : Check cases accoding to button position.
;Exit            : Exit the game on both calcs.
;Clear           : Clear the board on both calcs.
;
;The game can be played between a TI calc runnig IR_TTT
; (currently TI82, TI85, TI86, TI89 and TI92+).
;
;
;The game lacks of some essential features, because I made it
;very quickly:
;
;-There are no turns, a player can check as many cases
; as he wants.(be sure that the you opponents doesn't
; cheat ... ;)
;-The program doesn't detect if someone has won.
;-A piece can be erased by the other player.
;
;

	include "doorsos.h"
	include "graphlib.h"
	include "userlib.h"
   include "linelib.h"
   include "irlib.h"

	xdef	_ti89
   xdef  _ti92plus
   xdef	_main
	xdef	_comment



DrawLine macro
   move.w   \1,D0
   move.w   \2,D1
   move.w   \3,D2
   move.w   \4,D3
   move.l   #LCD_MEM,a0
   jsr      linelib::fline
   endm

_main:

   clr.b    $60000C
   bset.b   #6,$60000C      
   bclr.b   #0,$60000E        ; Set R1
   bclr.b   #1,$60000E        ; Set W1


ClearScreen:
   jsr      graphlib::clr_scr2

   SetFont  #2

   DrawLine #53,#1,#53,#90
   DrawLine #105,#1,#105,#90
   DrawLine #1,#30,#159,#30
   DrawLine #1,#60,#159,#60


WaitKey:
   tst.w    (doorsos::kb_vars+$1C)
   beq      CheckLines
   move.w   (doorsos::MaxHandles+$1E),d0
   clr.w    (doorsos::kb_vars+$1C)
   cmp.w    #264,d0
   beq      Exit                    ; Exit ?
   cmp.w    #263,d0                 ; Clear ?
   beq      Clear

   cmp.w    #57,d0                  ; 1...9 are between 49-57
   bhi      WaitKey
   cmp.w    #49,d0                  ; 1...9 are between 49-57
   blt      WaitKey

   sub.b    #49,d0                  ; Substract 49 from it.
   lea      KeyConv(PC),a2          ; A2 points to key conversion table.
   move.b   0(a2,d0),d3             ; Get TI8x key value.
   movem.l  d3-d4/a0,-(a7)
   jsr      irlib::PutByte          ; Send Key number.
   movem.l  (a7)+,d3-d4/a0

   move.b   #'X',d0
   bra      CheckMark   


CheckLines:
   btst.b   #2,$60000E
   bne      ReceiveData
   btst.b   #3,$60000E
   bne      ReceiveData
   bra      WaitKey
ReceiveData
   jsr      irlib::GetByte

CheckData:
   cmp.b    #$37,d3                 ; $37 is the ESC code on TI8x
   beq      RemoteExit              ; Other calc has exited?
   cmp.b    #$0F,d3                 ; $0F is the CLEAR code on TI8x
   beq      ClearScreen             ; Other calc has cleared?

   move.b   #'O',d0
   

CheckMark:
   cmp.b    #$22,d3
   bne      CheckPlace2
   move.l   #$0016004A,d2
   bra      DrawMark 
CheckPlace2
   cmp.b    #$1A,d3
   bne      CheckPlace3
   move.l   #$0048004A,d2
   bra      DrawMark 
CheckPlace3
   cmp.b    #$12,d3
   bne      CheckPlace4
   move.l   #$007A004A,d2
   bra      DrawMark 
CheckPlace4
   cmp.b    #$23,d3
   bne      CheckPlace5
   move.l   #$0016002B,d2
   bra      DrawMark 
CheckPlace5
   cmp.b    #$1B,d3
   bne      CheckPlace6
   move.l   #$0048002B,d2
   bra      DrawMark 
CheckPlace6
   cmp.b    #$13,d3
   bne      CheckPlace7
   move.l   #$007A002B,d2
   bra      DrawMark 
CheckPlace7
   cmp.b    #$24,d3
   bne      CheckPlace8
   move.l   #$0016000B,d2
   bra      DrawMark
CheckPlace8
   cmp.b    #$1C,d3
   bne      CheckPlace9
   move.l   #$0048000B,d2
   bra      DrawMark
CheckPlace9
   cmp.b    #$14,d3
   bne      CheckLines              ; If not equal return to main loop.
   move.l   #$007A000B,d2

DrawMark
   move.w   #4,-(a7)
   move.l   d2,-(a7)
   move.w   d0,-(a7)                ; d0 contains 'X' or 'O'
   jsr      userlib::DrawCharXY
   lea      8(a7),a7
   bra      CheckLines
   


Exit:
   move.b   #$37,d3                 ; $37 for compability with other TI8x
   jsr      irlib::PutByte
RemoteExit
   jsr      doorsos::reset_link
	rts

Clear:
   move.b   #$0F,d3                  ; $0F for compability with other TI8x
   jsr      irlib::PutByte
   bra      ClearScreen


KeyConv:                        ; To convert keys codes to other TI8x.
   dc.b $22
   dc.b $1A
   dc.b $12
   dc.b $23
   dc.b $1B
   dc.b $13
   dc.b $24
   dc.b $1C
   dc.b $14


_comment dc.b  "Tic-Tac-Toe using the IR Link",0
	end

